@@ -1,12 +1,8 @@
 #include "reconstruct.h"
 
 #include <Rcpp.h>
-#include <unistd.h>
 
-#include <ctime>
-#include <fstream>
-#include <iostream>
-#include <sstream>
+#include <chrono>
 #include <string>
 
 #include "confidence_cut.h"
@@ -14,8 +10,6 @@
 #include "utilities.h"
 
 using Rcpp::_;
-using Rcpp::as;
-using Rcpp::DataFrame;
 using Rcpp::List;
 using Rcpp::Rcout;
 using std::string;
@@ -24,53 +18,41 @@ using namespace miic::reconstruction;
 using namespace miic::structure;
 using namespace miic::utility;
 
-List empty_results() {
-  List result;
-  result = List::create(_["interrupted"] = true);
-  return (result);
-}
+List empty_results() { return List::create(_["interrupted"] = true); }
 
 // [[Rcpp::export]]
 List reconstruct(List input_data, List arg_list) {
   Environment environment(input_data, arg_list);
 
-  double startTime = get_wall_time();
-
   environment.memoryThreads = new MemorySpace[environment.n_threads];
   for (int i = 0; i < environment.n_threads; i++) {
     createMemorySpace(environment, environment.memoryThreads[i]);
   }
   createMemorySpace(environment, environment.m);
 
+  auto lap_start = getLapStartTime();
+  Rcout << "Search all pairs for unconditional independence relations...\n";
   // Initialize skeleton, find unconditional independence
-  if (!skeletonInitialization(environment)) {
-    List result =
-        List::create(
-            _["error"]       = "error during skeleton initialization",
-            _["interrupted"] = true);
-    return result;
-  }
+  if (!skeletonInitialization(environment)) return empty_results();
+  environment.exec_time.init += getLapInterval(lap_start);
 
-  long double spentTime = (get_wall_time() - startTime);
-  environment.exec_time.init = spentTime;
-  environment.exec_time.init_iter = spentTime;
-  if (environment.verbose) {
-    Rcout << "\n# ----> First contributing node elapsed time:" << spentTime
-              << "sec\n\n";
-  }
   BCC bcc(environment);
-  auto cycle_tracker = CycleTracker(environment);
+  CycleTracker cycle_tracker(environment);
   vector<vector<string>> orientations;
+  int iter_count{0};
+  bool is_consistent{false};
   do {
-    if (environment.consistent > 0) bcc.analyse();
-    // Save the neighbours in the status_prev structure
-    // and revert to the structure at the moment of initialization
+    if (environment.consistent != 0) bcc.analyse();
+    // Store current status in status_prev and revert to the structure at the
+    // moment of initialization
     for (int i = 0; i < environment.n_nodes; i++) {
       for (int j = 0; j < environment.n_nodes; j++) {
         environment.edges[i][j].status_prev = environment.edges[i][j].status;
         environment.edges[i][j].status = environment.edges[i][j].status_init;
       }
     }
+    lap_start = getLapStartTime();
+    Rcout << "Collect candidate separating nodes...\n";
     // If interrupted
     if (!firstStepIteration(environment, bcc)) return empty_results();
 
@@ -83,32 +65,40 @@ List reconstruct(List input_data, List arg_list) {
       if (environment.verbose) {
         Rcout << "\n# ---- Other Contributing node(s) ----\n\n";
       }
-      startTime = get_wall_time();
-
+      Rcout << "Search for conditional independence relations...\n";
       // If interrupted
       if (!skeletonIteration(environment)) return (empty_results());
-
-      long double spentTime = (get_wall_time() - startTime);
-      environment.exec_time.iter += spentTime;
-      environment.exec_time.init_iter += spentTime;
     }
+    environment.exec_time.iter += getLapInterval(lap_start);
 
     if (environment.n_shuffles > 0) {
-      startTime = get_wall_time();
-      Rcout << "Computing confidence cut with permutations..." << std::flush;
+      lap_start = getLapStartTime();
+      Rcout << "Compute confidence cut with permutations..." << std::flush;
       setConfidence(environment);
-      long double spentTime = (get_wall_time() - startTime);
-      environment.exec_time.cut += spentTime;
-      Rcout << " done.\n";
       confidenceCut(environment);
+      environment.exec_time.cut += getLapInterval(lap_start);
     }
     // Oriente edges for non-consistent/orientation consistent algorithm
     if (environment.orientation_phase && environment.numNoMore > 0 &&
         environment.consistent <= 1) {
+      lap_start = getLapStartTime();
+      Rcout << "Search for edge directions...\n";
       orientations = orientationProbability(environment);
+      environment.exec_time.ori += getLapInterval(lap_start);
     }
-    Rcout << "Number of edges: " << environment.numNoMore << std::endl;
-  } while (environment.consistent > 0 && !cycle_tracker.hasCycle());
+    if (environment.consistent != 0)
+      Rcout << "Iteration " << iter_count << ' ';
+    Rcout << "Number of edges: " << environment.numNoMore << '\n';
+    is_consistent = cycle_tracker.hasCycle();
+    if (is_consistent) {
+      Rcout << "cycle found of size " << cycle_tracker.getCycleSize() << '\n';
+      break;
+    }
+    if (++iter_count > environment.max_iteration) {
+      Rcout << "Iteration limit " << environment.max_iteration << " reached\n";
+      break;
+    }
+  } while (environment.consistent != 0);
 
   int union_n_edges = 0;
   for (int i = 1; i < environment.n_nodes; i++) {
@@ -122,16 +112,18 @@ List reconstruct(List input_data, List arg_list) {
 
   // skeleton consistent algorithm
   if (environment.numNoMore > 0 && environment.consistent == 2) {
+    lap_start = getLapStartTime();
     orientations = orientationProbability(environment);
+    environment.exec_time.ori += getLapInterval(lap_start);
     // Check inconsistency after orientation, add undirected edge to
     // pairs with inconsistent conditional independence.
     bcc.analyse();
     int n_inconsistency = 0;
-    std::vector<std::pair<int, int> > inconsistent_edges;
+    vector<std::pair<int, int>> inconsistent_edges;
     for (int i = 1; i < environment.n_nodes; i++) {
       for (int j = 0; j < i; j++) {
         const Edge& edge = environment.edges[i][j];
-        if (edge.status || bcc.is_consistent(i, j, edge.shared_info->ui_list))
+        if (edge.status || bcc.isConsistent(i, j, edge.shared_info->ui_list))
           continue;
         if (environment.verbose) {
           Rcout << environment.nodes[i].name << ",\t"
@@ -149,24 +141,22 @@ List reconstruct(List input_data, List arg_list) {
       environment.edges[k.first][k.second].shared_info->setUndirected();
     }
     Rcout << n_inconsistency << " inconsistent conditional independences"
-              << " found after orientation." << std::endl;
+          << " found after orientation.\n";
   }
 
-  vector<double> time;
-  time.push_back(environment.exec_time.init);
-  time.push_back(environment.exec_time.iter);
-  time.push_back(environment.exec_time.cut);
-  time.push_back(environment.exec_time.init_iter + environment.exec_time.cut);
-
-  List result;
-  result = List::create(
+  const auto& time = environment.exec_time;
+  List result = List::create(
       _["adj_matrix"]        = getAdjMatrix(environment),
       _["edges"]             = getEdgesInfoTable(environment),
       _["orientations.prob"] = orientations,
-      _["time"]              = time,
+      _["time"]              = vector<double>{
+        time.init, time.iter, time.cut, time.ori, time.getTotal()},
       _["interrupted"]       = false);
-  if (environment.consistent > 0) {
-    result.push_back(cycle_tracker.adj_matrices, "adj_matrices");
+  if (environment.consistent != 0) {
+    int size = is_consistent ? cycle_tracker.getCycleSize()
+                             : environment.max_iteration;
+    result.push_back(cycle_tracker.getAdjMatrices(size), "adj_matrices");
+    result.push_back(is_consistent, "is_consistent");
   }
 
   for (int i = 0; i < environment.n_threads; i++) {
@@ -196,11 +186,6 @@ bool CycleTracker::hasCycle() {
   for (auto it = range.first; it != range.second; ++it)
     iter_indices.push_back(it->second + 1);
   saveIteration();
-  if (n_saved > env_.max_iteration) {
-    Rcout << "Max number of iterations reached: " << env_.max_iteration
-              << '\n';
-    return true;
-  }
   if (no_cycle_found) return false;
   // Backtracking requires starting from the largest index first
   std::sort(iter_indices.begin(), iter_indices.end(), std::greater<int>());
@@ -212,43 +197,41 @@ bool CycleTracker::hasCycle() {
   // of iterations).
   vector<int> changed(env_.n_nodes * (env_.n_nodes - 1) / 2, 0);
   // backtracking over iteration to get changed_edges
-  int cycle_size = 0;
+  cycle_size = 0;
   for (const auto& iter : iterations_) {
     ++cycle_size;
     for (const auto& k : iter.changed_edges) {
       edges_union.insert(k.first);
-      // compare edge status in the previous iteration against the latest edge
-      // status
+      // compare the status in the previous iteration against the latest status
       std::pair<int, int> p = getEdgeIndex2D(k.first);
       changed[k.first] = (k.second != env_.edges[p.first][p.second].status);
     }
     if (iter.index != iter_indices.front()) continue;
     iter_indices.pop_front();
-    // if any edge has been changed
-    if (std::any_of(
-            changed.begin(), changed.end(), [](int j) { return j != 0; })) {
-      // no cycle
-      if (iter_indices.empty())
-        return false;
-      else
-        continue;
+    using std::none_of;
+    if (none_of(begin(changed), end(changed), [](int j) { return j != 0; })) {
+      for (auto& k : edges_union) {
+        std::pair<int, int> p = getEdgeIndex2D(k);
+        env_.edges[p.first][p.second].status = 1;
+        env_.edges[p.second][p.first].status = 1;
+        env_.edges[p.first][p.second].shared_info->setUndirected();
+      }
+      return true;
     }
-    for (auto& k : edges_union) {
-      std::pair<int, int> p = getEdgeIndex2D(k);
-      env_.edges[p.first][p.second].status = 1;
-      env_.edges[p.second][p.first].status = 1;
-      env_.edges[p.first][p.second].shared_info->setUndirected();
-    }
-
-    Rcout << "cycle found of size " << cycle_size << std::endl;
-    break;
+    if (iter_indices.empty()) return false;  // no cycle
   }
-  // fill the adj_matrices
-  for (auto i = iterations_.begin(), e = iterations_.begin() + cycle_size;
-       i != e; ++i) {
+  // Never reached since iter_indices.size() < iterations_.size() by definition
+  return false;
+}
+
+vector<vector<int>> CycleTracker::getAdjMatrices(int size) {
+  vector<vector<int>> adj_matrices;
+  for (auto i = iterations_.begin(), e = iterations_.begin() + size; i != e;
+      ++i) {
     adj_matrices.push_back(i->adj_matrix_1d);
   }
-  return true;
+  return adj_matrices;
 }
+
 }  // namespace reconstruction
 }  // namespace miic
